@@ -158,6 +158,8 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
         return
 
     if orca_state.activeScript:
+        msg = "ORCA: Active script is: %s" % orca_state.activeScript
+        debug.println(debug.LEVEL_INFO, msg, True)
         if orca_state.activeScript.utilities.isZombie(obj):
             msg = "ERROR: New locusOfFocus (%s) is zombie. Not updating." % obj
             debug.println(debug.LEVEL_INFO, msg, True)
@@ -221,6 +223,14 @@ def _processBrailleEvent(event):
 #                                                                      #
 ########################################################################
 
+def deviceChangeHandler(deviceManager, device):
+    """New keyboards being plugged in stomp on our changes to the keymappings, so we have to re-apply"""
+    source = device.get_source()
+    if source == Gdk.InputSource.KEYBOARD:
+        msg = "ORCA: Keyboard change detected, re-creating the xmodmap"
+        debug.println(debug.LEVEL_INFO, msg, True)
+        _createOrcaXmodmap()
+
 def updateKeyMap(keyboardEvent):
     """Unsupported convenience method to call sad hacks which should go away."""
 
@@ -248,32 +258,51 @@ def _setCapsLockAsOrcaModifier(enable):
     """Enable or disable use of the caps lock key as an Orca modifier key."""
     interpretCapsLineProg = re.compile(
         r'^\s*interpret\s+Caps[_+]Lock[_+]AnyOfOrNone\s*\(all\)\s*{\s*$', re.I)
-    capsModLineProg = re.compile(
-        r'^\s*action\s*=\s*NoAction\s*\(\s*\)\s*;\s*$', re.I)
     normalCapsLineProg = re.compile(
         r'^\s*action\s*=\s*LockMods\s*\(\s*modifiers\s*=\s*Lock\s*\)\s*;\s*$', re.I)
+    interpretShiftLineProg = re.compile(
+        r'^\s*interpret\s+Shift[_+]Lock[_+]AnyOf\s*\(\s*Shift\s*\+\s*Lock\s*\)\s*{\s*$', re.I)
+    normalShiftLineProg = re.compile(
+        r'^\s*action\s*=\s*LockMods\s*\(\s*modifiers\s*=\s*Shift\s*\)\s*;\s*$', re.I)
+    disabledModLineProg = re.compile(
+        r'^\s*action\s*=\s*NoAction\s*\(\s*\)\s*;\s*$', re.I)
     normalCapsLine = '        action= LockMods(modifiers=Lock);'
-    capsModLine =    '        action= NoAction();'
+    normalShiftLine = '        action= LockMods(modifiers=Shift);'
+    disabledModLine = '        action= NoAction();'
     lines = _originalXmodmap.decode('UTF-8').split('\n')
     foundCapsInterpretSection = False
+    foundShiftInterpretSection = False
+    modified = False
     for i, line in enumerate(lines):
-        if not foundCapsInterpretSection:
+        if not foundCapsInterpretSection and not foundShiftInterpretSection:
             if interpretCapsLineProg.match(line):
                 foundCapsInterpretSection = True
-        else:
+            elif interpretShiftLineProg.match(line):
+                foundShiftInterpretSection = True
+        elif foundCapsInterpretSection:
             if enable:
                 if normalCapsLineProg.match(line):
-                    lines[i] = capsModLine
-                    _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
-                    return
+                    lines[i] = disabledModLine
+                    modified = True
             else:
-                if capsModLineProg.match(line):
+                if disabledModLineProg.match(line):
                     lines[i] = normalCapsLine
-                    _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
-                    return
+                    modified = True
             if line.find('}'):
-                # Failed to find the line we need to change
-                return
+                foundCapsInterpretSection = False
+        else: # foundShiftInterpretSection
+            if enable:
+                if normalShiftLineProg.match(line):
+                    lines[i] = disabledModLine
+                    modified = True
+            else:
+                if disabledModLineProg.match(line):
+                    lines[i] = normalShiftLine
+                    modified = True
+            if line.find('}'):
+                foundShiftInterpretSection = False
+    if modified:
+        _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
 
 def _createOrcaXmodmap():
     """Makes an Orca-specific Xmodmap so that the keys behave as we
@@ -283,7 +312,8 @@ def _createOrcaXmodmap():
     global _capsLockCleared
 
     cmd = []
-    if "Caps_Lock" in settings.orcaModifierKeys:
+    if "Caps_Lock" in settings.orcaModifierKeys \
+       or "Shift_Lock" in settings.orcaModifierKeys:
         _setCapsLockAsOrcaModifier(True)
         _capsLockCleared = True
     elif _capsLockCleared:
@@ -554,6 +584,15 @@ def start(registry, cacheValues):
     if cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
+    # Event handlers for input devices being plugged in/unplugged.
+    # Used to re-create the Xmodmap when a new keyboard is plugged in.
+    # Necessary, because plugging in a new keyboard resets the Xmodmap
+    # and stomps our changes
+    display = Gdk.Display.get_default()
+    devmanager=display.get_device_manager()
+    devmanager.connect("device-added", deviceChangeHandler)
+    devmanager.connect("device-removed", deviceChangeHandler)
+
     Gdk.notify_startup_complete()
     msg = 'ORCA: Startup complete notification made'
     debug.println(debug.LEVEL_INFO, msg, True)
