@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2018 Igalia, S.L.
+# Copyright 2018-2019 Igalia, S.L.
 #
 # Author: Joanmarie Diggs <jdiggs@igalia.com>
 #
@@ -28,7 +28,7 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2018 Igalia, S.L."
+__copyright__ = "Copyright (c) 2018-2019 Igalia, S.L."
 __license__   = "LGPL"
 
 import pyatspi
@@ -64,6 +64,21 @@ class SpeechGenerator(web.SpeechGenerator):
 
         return super()._generateListBoxItemWidgets(obj, **args)
 
+    def _generateLabelOrName(self, obj, **args):
+        if obj.getRole() == pyatspi.ROLE_FRAME:
+            document = self._script.utilities.activeDocument(obj)
+            if document and not self._script.utilities.documentFrameURI(document):
+                # Eliminates including "untitled" in the frame name.
+                return super()._generateLabelOrName(obj.parent)
+
+        return super()._generateLabelOrName(obj)
+
+    def _generateRoleName(self, obj, **args):
+        if self._script.utilities.isListItemMarker(obj):
+            return []
+
+        return super()._generateRoleName(obj, **args)
+
     def generateSpeech(self, obj, **args):
         if self._script.utilities.inDocumentContent(obj):
             return super().generateSpeech(obj, **args)
