@@ -32,6 +32,7 @@ __license__   = "LGPL"
 
 import pyatspi
 import re
+import time
 
 from orca import debug
 from orca import orca_state
@@ -42,6 +43,9 @@ class Utilities(web.Utilities):
 
     def __init__(self, script):
         super().__init__(script)
+        self._lastAutoTextObjectEvent = None
+        self._lastAutoTextInputEvent = None
+        self._lastAutoTextEventTime = 0
 
     def _attemptBrokenTextRecovery(self, obj, **args):
         boundary = args.get('boundary')
@@ -175,6 +179,20 @@ class Utilities(web.Utilities):
         if uri.startswith("moz-extension"):
             return True
 
+        if not uri and "pixels" in name:
+            return True
+
+        return False
+
+    def _objectMightBeBogus(self, obj):
+        if not obj:
+            return False
+
+        if obj.getRole() == pyatspi.ROLE_SECTION and obj.parent.getRole() == pyatspi.ROLE_FRAME:
+            msg = "GECKO: %s is believed to be a bogus object" % obj
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return True
+
         return False
 
     def canBeActiveWindow(self, window, clearCache=False):
@@ -319,3 +337,22 @@ class Utilities(web.Utilities):
         label = labels[0]
         label.clearCache()
         return label.name
+
+    def isAutoTextEvent(self, event):
+        if not super().isAutoTextEvent(event):
+            return False
+
+        if self.inDocumentContent(event.source):
+            return True
+
+        if self.treatAsDuplicateEvent(self._lastAutoTextObjectEvent, event) \
+           and time.time() - self._lastAutoTextEventTime < 0.5 \
+           and orca_state.lastInputEvent.isReleaseFor(self._lastAutoTextInputEvent):
+            msg = "GECKO: Event believed to be duplicate auto text event."
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return False
+
+        self._lastAutoTextObjectEvent = event
+        self._lastAutoTextInputEvent = orca_state.lastInputEvent
+        self._lastAutoTextEventTime = time.time()
+        return True
