@@ -33,6 +33,7 @@ import pyatspi.utils as utils
 import orca.scripts.default as default
 import orca.cmdnames as cmdnames
 import orca.debug as debug
+import orca.eventsynthesizer as eventsynthesizer
 import orca.guilabels as guilabels
 import orca.input_event as input_event
 import orca.messages as messages
@@ -148,6 +149,7 @@ class Script(default.Script):
                 structural_navigation.StructuralNavigation.CHUNK,
                 structural_navigation.StructuralNavigation.CLICKABLE,
                 structural_navigation.StructuralNavigation.COMBO_BOX,
+                structural_navigation.StructuralNavigation.CONTAINER,
                 structural_navigation.StructuralNavigation.ENTRY,
                 structural_navigation.StructuralNavigation.FORM_FIELD,
                 structural_navigation.StructuralNavigation.HEADING,
@@ -347,11 +349,7 @@ class Script(default.Script):
 
         phrase = self.utilities.substring(obj, startOffset, endOffset)
         if len(phrase) and phrase != "\n":
-            if phrase.isupper():
-                voice = self.voices[settings.UPPERCASE_VOICE]
-            else:
-                voice = self.voices[settings.DEFAULT_VOICE]
-
+            voice = self.speechGenerator.voice(string=phrase)
             phrase = self.utilities.adjustForRepeats(phrase)
             links = [x for x in obj if x.getRole() == pyatspi.ROLE_LINK]
             if links:
@@ -514,8 +512,10 @@ class Script(default.Script):
         if obj.getRole() == pyatspi.ROLE_LINK:
             obj = obj.parent
 
-        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        document = utils.findAncestor(obj, lambda x: x.getRole() in docRoles)
+        if self.utilities.isDocument(obj):
+            document = obj
+        else:
+            document = self.utilities.getContainingDocument(obj)
         if not document or document.getState().contains(pyatspi.STATE_BUSY):
             return
 
@@ -531,18 +531,21 @@ class Script(default.Script):
         if sayAllStyle == settings.SAYALL_STYLE_SENTENCE:
             boundary = pyatspi.TEXT_BOUNDARY_SENTENCE_START
 
+        voices = _settingsManager.getSetting('voices')
+        systemVoice = voices.get(settings.SYSTEM_VOICE)
+
         self._inSayAll = True
         offset = textObjs[0].queryText().caretOffset
         for textObj in textObjs:
             textSegments = self.getTextSegments(textObj, boundary, offset)
-            roleInfo = self.speechGenerator.getRoleName(textObj)
-            if roleInfo:
-                roleName, voice = roleInfo
-                textSegments.append([roleName, 0, -1, voice])
+            roleName = self.speechGenerator.getRoleName(textObj)
+            if roleName:
+                textSegments.append([roleName, 0, -1, systemVoice])
 
             for (string, start, end, voice) in textSegments:
                 context = speechserver.SayAllContext(textObj, string, start, end)
                 self._sayAllContexts.append(context)
+                eventsynthesizer.scrollIntoView(obj, start, end)
                 yield [context, voice]
 
             offset = 0
