@@ -509,6 +509,12 @@ class Script(default.Script):
 
         return super().skipObjectEvent(event)
 
+    def presentationInterrupt(self):
+        super().presentationInterrupt()
+        msg = "WEB: Flushing live region messages"
+        debug.println(debug.LEVEL_INFO, msg, True)
+        self.liveRegionManager.flushMessages()
+
     def consumesKeyboardEvent(self, keyboardEvent):
         """Returns True if the script will consume this keyboard event."""
 
@@ -2123,8 +2129,6 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return True
 
-        # TODO - JD: As an experiment, we're stopping these at the event manager.
-        # If that works, this can be removed.
         if self.utilities.eventIsEOCAdded(event):
             msg = "WEB: Ignoring: Event was for embedded object char"
             debug.println(debug.LEVEL_INFO, msg, True)
