@@ -49,7 +49,7 @@ class Player:
 
         if not _gstreamerAvailable:
             msg = 'SOUND ERROR: Gstreamer is not available'
-            debug.println(debug.LEVEL_INFO, mSG, True)
+            debug.println(debug.LEVEL_INFO, msg, True)
             return
 
         self.init()
@@ -104,6 +104,9 @@ class Player:
         if self._initialized:
             return
 
+        if not _gstreamerAvailable:
+            return
+
         self._player = Gst.ElementFactory.make('playbin', 'player')
         bus = self._player.get_bus()
         bus.add_signal_watch()
@@ -116,6 +119,9 @@ class Player:
 
         self._source = Gst.ElementFactory.make('audiotestsrc', 'src')
         self._sink = Gst.ElementFactory.make('autoaudiosink', 'output')
+        if self._source is None or self._sink is None:
+            return
+
         self._pipeline.add(self._source)
         self._pipeline.add(self._sink)
         self._source.link(self._sink)
