@@ -221,24 +221,28 @@ class SpeechServer(speechserver.SpeechServer):
         if acss_family is None:
             acss_family = {}
 
-        familyLocale = acss_family.get(speechserver.VoiceFamily.LOCALE)
-        if not familyLocale:
+        language = acss_family.get(speechserver.VoiceFamily.LANG)
+        dialect = acss_family.get(speechserver.VoiceFamily.DIALECT)
+
+        if not language:
             import locale
             familyLocale, encoding = locale.getdefaultlocale()
 
-        language, dialect = '', ''
-        if familyLocale:
-            localeValues = familyLocale.split('_')
-            language = localeValues[0]
-            if len(localeValues) == 2:
-                dialect = localeValues[1]
+            language, dialect = '', ''
+            if familyLocale:
+                localeValues = familyLocale.split('_')
+                language = localeValues[0]
+                if len(localeValues) == 2:
+                    dialect = localeValues[1]
 
         return language, dialect
 
     def _set_family(self, acss_family):
         lang, dialect = self._get_language_and_dialect(acss_family)
-        if len(lang) == 2:
-            self._send_command(self._client.set_language, lang)
+        self._send_command(self._client.set_language, lang)
+        if dialect:
+            # Try to set precise dialect
+            self._send_command(self._client.set_language, lang + '-' + dialect)
 
         try:
             # This command is not available with older SD versions.
@@ -349,7 +353,10 @@ class SpeechServer(speechserver.SpeechServer):
         # string offsets
         # Note: we need to do this before disturbing the text offsets
         # Note2: we assume that text mangling below leave U+E000 untouched
+        last_begin = None
+        last_end = None
         marks_offsets = []
+        marks_endoffsets = []
         marked_text = ""
 
         for i in range(len(text)):
@@ -358,14 +365,24 @@ class SpeechServer(speechserver.SpeechServer):
                 # Original text already contains U+E000. But syntheses will not
                 # know what to do of it anyway, so discard it
                 continue
+
+            if not c.isspace() and last_begin == None:
+                # Word begin
+                marked_text += '\ue000'
+                last_begin = i
+
+            if c.isspace() and last_begin != None:
+                # Word end, add a mark
+                marks_offsets.append(last_begin)
+                marks_endoffsets.append(i)
+                last_begin = None
+
             marked_text += c
 
-            if (c == ' ' or c == '\u00a0') \
-               and i < len(text) - 1 \
-               and text[i + 1] != ' ' and text[i + 1] != '\u00a0':
-                # Word separation, add a mark
-                marks_offsets.append(i + 1)
-                marked_text += '\ue000'
+        if last_begin != None:
+            # Finished with a word
+            marks_offsets.append(last_begin)
+            marks_endoffsets.append(i + 1)
 
         text = marked_text
 
@@ -396,7 +413,7 @@ class SpeechServer(speechserver.SpeechServer):
                     msg = "%uth U+E000 does not have corresponding index" % i
                     debug.println(debug.LEVEL_WARNING, msg, True)
                 else:
-                    ssml += '<mark name="%u"/>' % marks_offsets[i]
+                    ssml += '<mark name="%u:%u"/>' % (marks_offsets[i], marks_endoffsets[i])
                 i += 1
             # Disable for now, until speech dispatcher properly parses them (version 0.8.9 or later)
             #elif c == '"':
@@ -435,13 +452,21 @@ class SpeechServer(speechserver.SpeechServer):
                 t = self._CALLBACK_TYPE_MAP[callbackType]
                 if t == speechserver.SayAllContext.PROGRESS:
                     if index_mark:
-                        context.currentOffset = context.startOffset + int(index_mark)
-                        msg = "SPEECH DISPATCHER: Got mark %d / %d-%d" % (context.currentOffset, context.startOffset, context.endOffset)
-                        debug.println(debug.LEVEL_INFO, msg, True)
+                        index = index_mark.split(':')
+                        if len(index) >= 2:
+                            start, end = index[0:2]
+                            context.currentOffset = context.startOffset + int(start)
+                            context.currentEndOffset = context.startOffset + int(end)
+                            msg = "SPEECH DISPATCHER: Got mark %d:%d / %d-%d" % \
+                                (context.currentOffset, context.currentEndOffset, \
+                                 context.startOffset, context.endOffset)
+                            debug.println(debug.LEVEL_INFO, msg, True)
                     else:
                         context.currentOffset = context.startOffset
+                        context.currentEndOffset = None
                 elif t == speechserver.SayAllContext.COMPLETED:
                     context.currentOffset = context.endOffset
+                    context.currentEndOffset = None
                 GLib.idle_add(orca_callback, context, t)
                 if t == speechserver.SayAllContext.COMPLETED:
                     GLib.idle_add(self._say_all, iterator, orca_callback)
@@ -500,11 +525,11 @@ class SpeechServer(speechserver.SpeechServer):
         from locale import getlocale, LC_MESSAGES
         locale = getlocale(LC_MESSAGES)[0]
         if locale is None or locale == 'C':
-            lang = None
-            dialect = None
+            locale_language = None
         else:
-            lang, dialect = locale.split('_')
-        voices = ((self._default_voice_name, lang, None),)
+            locale_lang, locale_dialect = locale.split('_')
+            locale_language = locale_lang + '-' + locale_dialect
+        voices = ()
         try:
             # This command is not available with older SD versions.
             list_synthesis_voices = self._client.list_synthesis_voices
@@ -515,12 +540,33 @@ class SpeechServer(speechserver.SpeechServer):
                 voices += self._send_command(list_synthesis_voices)
             except:
                 pass
-        families = [speechserver.VoiceFamily({ \
+
+        default_lang = ""
+        if locale_language:
+            # Check whether how it appears in the server list
+            for name, lang, variant in voices:
+                if lang == locale_language:
+                    default_lang = locale_language
+                    break
+            if not default_lang:
+                for name, lang, variant in voices:
+                    if lang == locale_lang:
+                        default_lang = locale_lang
+            if not default_lang:
+                default_lang = locale_language
+
+        voices = ((self._default_voice_name, default_lang, None),) + voices
+
+        families = []
+        for name, lang, variant in voices:
+
+            families.append(speechserver.VoiceFamily({ \
               speechserver.VoiceFamily.NAME: name,
               #speechserver.VoiceFamily.GENDER: speechserver.VoiceFamily.MALE,
-              speechserver.VoiceFamily.DIALECT: dialect,
-              speechserver.VoiceFamily.LOCALE: lang})
-                    for name, lang, dialect in voices]
+              speechserver.VoiceFamily.LANG: lang.partition("-")[0],
+              speechserver.VoiceFamily.DIALECT: lang.partition("-")[2],
+              speechserver.VoiceFamily.VARIANT: variant}))
+
         return families
 
     def speak(self, text=None, acss=None, interrupt=True):
